"""Build an ingestion manifest file"""
import json
import logging
import re
import sys
import tarfile
from pathlib import Path
from typing import List, Tuple, Dict

# pylint: disable=E0401, E0402,R1721
import pendulum
from pendulum import DateTime

from .utilities import (
    MANIFEST_NAME_BASE,
    MANIFEST_NAME_EXT,
    ARTIFACT_NAME,
    ARTIFACT_EXT,
    WEBLOG,
    SCIENCE_PRODUCT_PATTERN,
    EvlaCalInputScienceProduct,
    EvlaCalInputGroup,
    EvlaCalOutputScienceProduct,
    EvlaCalOutputGroup,
)

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


class EvlaCalIngestionManifestWriter:
    """For building ingestion manifests"""

    # (science product type is always EVLA_CAL)
    def __init__(self, ingest_path: Path):
        self.ingest_path = ingest_path
        self.input_group = self.output_group = self.infiles = None

    def write_evla_cal_manifest(self, locator: str) -> Tuple[Path, List[Path]]:
        """
        Write the manifest and associated ingestion files

        :param locator: science product locator
        :return:
        """

        self.infiles = [file for file in self.ingest_path.iterdir()]
        self.input_group = EvlaCalInputGroup(EvlaCalInputScienceProduct(locator))
        self.output_group = self._make_evla_cal_output_group()

        # Pull out the manifest content and stringify it
        manifest_content = json.dumps(self.content(), indent=4)

        manifest_filename = self.manifest_filename()
        # Write the manifest to the staging area
        staging_manifest = Path(self.ingest_path / manifest_filename)
        with open(staging_manifest, "w") as out:
            out.write(f"{manifest_content}\n")

        # Get all the files we'll need
        addl_ingestion_files = self.find_additional_ingestion_files()

        # Write the artifacts tar.
        artifacts_tar = self.write_ingestion_artifacts_tar(staging_manifest, addl_ingestion_files)
        addl_ingestion_files.append(artifacts_tar)

        # return a Path explicitly; LocalPath won't work
        return Path(staging_manifest), addl_ingestion_files

    def write_ingestion_artifacts_tar(
        self, manifest_file: Path, ingestion_files: List[Path]
    ) -> Path:
        """
        Take the list of files and build a tar for inclusion into the archive.
        This happens in the staging area for ingestion.
        The EVLA CAL tar will contain just the manifest

        :param manifest_file: the ingestion manifest
        :param ingestion_files: all the files needed for ingestion
        :return: a .tar archive of the ingestion artifacts
        """
        ingestion_artifacts_fn = self.ingest_path / "ingestion_artifacts.tar"
        with tarfile.open(ingestion_artifacts_fn, "w") as ingestion_artifacts_tar:
            for file in ingestion_files:
                ingestion_artifacts_tar.add(file)

            # include the manifest
            if manifest_file not in ingestion_files:
                ingestion_artifacts_tar.add(manifest_file)

        return ingestion_artifacts_fn

    def _make_evla_cal_output_group(self):
        """
        Create the JSON for the "output-group" section of the manifest.
        An EVLA CAL OutputGroup contains a science product(s) and an ancillary product
        (the weblog)

        :return: manifest output group
        """
        sp_tar = self._find_science_product_tar()
        science_product = EvlaCalOutputScienceProduct(str(sp_tar))
        weblog = Path(self.ingest_path / WEBLOG)
        if weblog.exists():
            return EvlaCalOutputGroup(science_product, weblog)

        return EvlaCalOutputGroup(science_product, None)

    def content(self) -> Dict[str, str]:
        """
        Accessor for manifest content

        :return: manifest as dict
        """

        params = {
            "reingest": "false",
            "ngas_ingest": "false",
            "calibrate": "false",
            "ingestion_path": str(self.ingest_path),
        }

        json_out = dict(
            parameters=params,
            input_group=self.input_group.__json__(),
            output_group=self.output_group.__json__(),
            # associate_group=self.associate_group.__json__(), # TODO when we need it
            ingestion_path=str(self.ingest_path),
        )
        return json_out

    def _find_science_product_tar(self) -> Path:
        """
        A calibration ingestion staging dir should have ONE science product tar; ignore any others

        :return:
        """
        files = [file for file in self.ingest_path.iterdir() if file.is_file]
        for file in files:
            if re.match(SCIENCE_PRODUCT_PATTERN, file.name):
                return file

        raise FileNotFoundError(f"no science product found at {self.ingest_path}")

    @staticmethod
    def format_timestamp(datetime: DateTime) -> str:
        """
        Format the current time as
            2021_07_01T13_49_17.237

        :param datetime: current timestamp
        :return: timestamp suitable for ingestion manifest filename
        """

        return datetime.format("YYYY_MM_DDThh_mm_ss.SSS")

    @staticmethod
    def manifest_filename() -> str:
        """
        Build unique manifest filename in standard format.

        :return: the filename
        """
        current_time = pendulum.now()
        timestamp = EvlaCalIngestionManifestWriter.format_timestamp(current_time)
        return f"{MANIFEST_NAME_BASE}{timestamp}{MANIFEST_NAME_EXT}"

    @staticmethod
    def artifact_filename() -> str:
        """
        Build manifest and artifacts filenames.

        :return: the filenames
        """
        current_time = pendulum.now()
        timestamp = EvlaCalIngestionManifestWriter.format_timestamp(current_time)
        artifact_filename = f"{ARTIFACT_NAME}{timestamp}{ARTIFACT_EXT}"
        return artifact_filename

    def find_additional_ingestion_files(self) -> List[Path]:
        """
        Gather the files required for ingestion.
        There won't be any for EVLA CAL ingestion; this is just a placeholder.

        :return: ingestion inputs
        """

        return []

        # TODO: we'll have extra information for other ingestion types
        # coll_files = aux_files = []
        #
        # if self.additional_metadata is not None:
        #     addl_md = json.loads(self.additional_metadata)
        #     aux_files.append(addl_md["filename"])
        #
        # if self.collection_metadata is not None:
        #     coll_md = json.loads(self.collection_metadata)
        #     coll_files.append(coll_md["filename"])
        #
        # aux_files += coll_files
        #
        # # be sure to add the manifest itself
        # aux_files.append(self.manifest_filename)
        #
        # return list(set(aux_files))
