""" Miscellaneous manifest-building tests """

import json
import logging
import shutil
import sys
from pathlib import Path

# pylint: disable=E0401, E0402, R1721

import pytest

from ingest_envoy.ingestion_manifest import IngestionManifest
from ingest_envoy.utilities import ScienceProductType, IngestionManifestKey
from .conftest import populate_fake_ingest_path, WANTED_FILENAMES, UNWANTED

logger = logging.getLogger(IngestionManifest.__name__)
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


def test_entry_point_for_evla_cal(ingest_path: Path):
    """
    Confirm that the ingestion launcher entrypoint functions as expected.

    :param ingest_path: fake tmp ingestion path
    :return:
    """
    populate_fake_ingest_path(ingest_path)
    manifest = IngestionManifest(
        str(ingest_path), ScienceProductType.EVLA_CAL.value, "uid://evla/calibration/meeniemyniemoe"
    )
    manifest.create()

    files = [file for file in ingest_path.iterdir()]
    manifest_file = [file for file in files if file.name.endswith(".json")][0]
    assert manifest_file.exists()

    assert len(files) == len(WANTED_FILENAMES) + len(UNWANTED) + 2

    # make sure manifest_file contains an IngestionManifest
    with open(manifest_file, "r") as out:
        manifest_content = dict(json.load(out).items())

    for key in ["parameters", "input_group", "output_group", "ingestion_path"]:
        assert key in manifest_content.keys()

    input_group = manifest_content[IngestionManifestKey.INPUT_GROUP.value]
    assert isinstance(input_group, dict)
    assert len(input_group) == 1

    output_group = manifest_content[IngestionManifestKey.OUTPUT_GROUP.value]
    assert isinstance(output_group, dict)
    for key, val in output_group.items():
        val = val.replace("'", '"', len(val))
        sci_prods = json.loads(val)

    assert isinstance(sci_prods, list)
    assert len(sci_prods) == 1

    ancillary_products = output_group[IngestionManifestKey.ANCILLARY_PRODUCTS.value]
    assert ancillary_products[0] == "["
    assert ancillary_products[-1] == "]"
    assert "weblog" in ancillary_products
    assert "type" in ancillary_products
    assert "filename" in ancillary_products

    a_prods = json.loads(ancillary_products.replace("'", '"', len(ancillary_products)))
    assert isinstance(a_prods, list)
    a_prods = a_prods[0]
    assert isinstance(a_prods, dict)
    for key, val in a_prods.items():
        assert isinstance(val, dict)
        a_prods = val
    assert len(a_prods) == 2

    shutil.rmtree(ingest_path)


@pytest.mark.skip("TODO: test_builds_realfast_sdm_manifest")
def test_builds_realfast_sdm_manifest():
    """
    Have we built a well-formed RealFast product ingestion manifest?

    :return:
    """
    raise NotImplementedError


@pytest.mark.skip("TODO: test_builds_vlass_quicklook_manifest")
def test_builds_vlass_quicklook_manifest():
    """
    Have we built a well-formed VLASS quicklook product ingestion manifest?

    :return:
    """
    raise NotImplementedError


@pytest.mark.skip("TODO: test_builds_gbt_manifest")
def test_builds_gbt_manifest():
    """
    Not yet in use, but sooner or later we'll need to create manifests
    for ingestion of GBT products.

    :return:
    """
    raise NotImplementedError


@pytest.mark.skip("TODO: test_builds_lvla_eb_manifest")
def test_builds_lvla_eb_manifest():
    """
    Not yet in use, but one of these days we'll need to create manifests
    for ingestion of LVLA execution blocks.

    :return:
    """
    raise NotImplementedError
